import 'package:dio/dio.dart';
import 'package:flutter/foundation.dart';
import 'package:lori_sdk/data/lori_api.dart';
import 'package:lori_sdk/data/models/banner.dart';
import 'package:lori_sdk/data/models/product/product.dart';
import 'package:lori_sdk/data/models/user_promotions/userPromotion.dart';

class SalesMarketingApi {
  static const GET_BANNERS = "sales/marketing/banners";

  static const GET_FAVORITES = "/sales/marketing/buyers/favorites";

  static const GET_USER_PROMOTIONS = "sales/marketing/buyers/promotions";
  static const ACTIVATE_USER_PROMOTION = "sales/marketing/buyers/promotions/active";


  Dio _requester;

  SalesMarketingApi({@required Dio requester}) : this._requester = requester;



  ///
  /// region Banners
  ///

  /// Retrieve all banners for the user
  Future<ApiCallback<List<Banner>>> getBanners({BannerType type = BannerType.carousel}) async {
    try {
      Map<String, dynamic> queryParameters = {
        'type': type.index
      };
      var response = await _requester.get(GET_BANNERS,
          queryParameters: queryParameters,
          options: Options(responseType: ResponseType.json));
      var responseData = response.data as List;
      var banners = List<Banner>();
      for (var row in responseData) {
        banners.add(Banner.fromJson(row));
      }
      return ApiCallback<List<Banner>>(isSuccessful: true, resultData: banners);
    } on DioError catch (e) {
      return ApiCallback<List<Banner>>().defaultErrorCallback(e);
    }
  }

  ///
  /// endregion
  ///

  ///
  /// region Favorites
  ///

  /// Retrieve all products favorited by the user
  Future<ApiCallback<List<Product>>> getFavoriteProducts() async {
    try {
      var response = await _requester.get(GET_FAVORITES,
          options: Options(responseType: ResponseType.json));
      var responseData = response.data as List;
      var products = List<Product>();
      for (var row in responseData) {
        products.add(Product.fromJson(row));
      }
      return ApiCallback<List<Product>>(
          isSuccessful: true, resultData: products);
    } on DioError catch (e) {
      return ApiCallback<List<Product>>().defaultErrorCallback(e);
    }
  }

  ///
  /// endregion
  ///


  ///
  /// region User Promotions
  ///

  /// Load all users promotions for the current user.
  Future<ApiCallback<List<UserPromotion>>> getUserPromotions() async {
    try {
      var response = await _requester.get(
          GET_USER_PROMOTIONS,
          options: Options(responseType: ResponseType.json)
      );
      var responseData = response.data as List;
      var promotions = List<UserPromotion>();
      for (var row in responseData) {
        promotions.add(UserPromotion.fromJson(row));
      }
      // Callback invocation
      return ApiCallback<List<UserPromotion>>(isSuccessful: true, resultData: promotions);
    } on DioError catch (e) {
      return ApiCallback<List<UserPromotion>>().defaultErrorCallback(e);
    }
  }

  /// Activate a promotion for the current user give it's id.
  Future<ApiCallback<void>> activateUserPromotion(String externalId) async {
    try {
      Map<String, String> body = {
        "partnerPromotionId": externalId,
      };
      await _requester.post(
          ACTIVATE_USER_PROMOTION,
          queryParameters: body,
          options: Options(responseType: ResponseType.json));
      // Callback invocation
      return ApiCallback<void>(isSuccessful: true);
    } on DioError catch (e) {
      return ApiCallback<void>().defaultErrorCallback(e);
    }
  }

  ///
  /// User Promotions part. All methods related to User Promotions module.
  /// endregion
  ///
}